@@ -35,6 +35,8 @@
 #define IWD_AP_INTERFACE "net.connman.iwd.AccessPoint"
 #define IWD_ADHOC_INTERFACE "net.connman.iwd.AdHoc"
 #define IWD_STATION_INTERFACE "net.connman.iwd.Station"
+#define IWD_P2P_INTERFACE "net.connman.iwd.p2p.Device"
+#define IWD_P2P_PEER_INTERFACE "net.connman.iwd.p2p.Peer"
 
 #define IWD_BASE_PATH "/net/connman/iwd"
 #define IWD_AGENT_MANAGER_PATH IWD_BASE_PATH
