@@ -204,8 +204,6 @@ static void scan_request_triggered(struct l_genl_msg *msg, void *userdata)
 	struct scan_request *sr = l_queue_peek_head(sc->requests);
 	int err;
 
-	l_debug("");
-
 	sc->start_cmd_id = 0;
 
 	err = l_genl_msg_get_error(msg);
@@ -548,6 +546,9 @@ static uint32_t scan_common(uint64_t wdev_id, bool passive,
 	if (!l_queue_isempty(sc->requests))
 		goto done;
 
+	if (sc->suspended)
+		goto done;
+
 	if (sc->state != SCAN_STATE_NOT_RUNNING)
 		goto done;
 
@@ -841,6 +842,9 @@ static bool start_next_scan_request(struct scan_context *sc)
 	if (sc->state != SCAN_STATE_NOT_RUNNING)
 		return true;
 
+	if (sc->start_cmd_id || sc->get_scan_cmd_id)
+		return true;
+
 	while (sr) {
 		if (!scan_request_send_trigger(sc, sr))
 			return true;
@@ -879,7 +883,7 @@ static bool scan_parse_vendor_specific(struct scan_bss *bss, const void *data,
 
 /*
  * Fully parses the Advertisement Protocol Element. The only thing being looked
- * for is the ANQP protocol ID, but this could be burried behind several other
+ * for is the ANQP protocol ID, but this could be buried behind several other
  * advertisement tuples so the entire IE may need to be parsed.
  */
 static bool scan_parse_advertisement_protocol(struct scan_bss *bss,
@@ -1667,7 +1671,7 @@ static void scan_notify(struct l_genl_msg *msg, void *user_data)
 			/* An external scan may have flushed our results */
 			if (sc->started && scan_parse_flush_flag_from_msg(msg))
 				scan_finished(sc, -EAGAIN, NULL, sr);
-			else if (sr && !sc->start_cmd_id)
+			else
 				send_next = true;
 
 			sr = NULL;
@@ -1718,7 +1722,7 @@ static void scan_notify(struct l_genl_msg *msg, void *user_data)
 			sc->triggered = false;
 
 			scan_finished(sc, -ECANCELED, NULL, sr);
-		} else if (sr && !sc->start_cmd_id && !sc->get_scan_cmd_id) {
+		} else {
 			/*
 			 * If this was an external scan that got aborted
 			 * we may be able to now queue our own scan although
